import { and, count, countDistinct, desc, eq, sql, type SQL } from 'drizzle-orm';
import type { IPAddressListEntry, IPAddressQuery } from './types';
import { DB, ipAddress, ipList, ipUser, user } from '../drizzle';
import type { PaginationMeta, Paginated } from '$lib/types';

export class IPAddressesAdmin {
	static async filterSearchIpAddresses(
		search: IPAddressQuery & {
			limit?: number;
			offset?: number;
		}
	) {
		const limit = search.limit || 20;
		const rowCount = await IPAddressesAdmin.ipCount(search);

		const ipAddressSubquery = DB.drizzle
			.select({ id: ipAddress.id })
			.from(ipAddress)
			.leftJoin(ipUser, eq(ipAddress.id, ipUser.ipAddressId))
			.leftJoin(user, eq(user.id, ipUser.userId))
			.limit(limit)
			.offset(search.offset || 0)
			.where(IPAddressesAdmin.getIpAddressWhere(search))
			.orderBy(desc(ipAddress.updated_at))
			.as('ipAddressSubquery');

		const list: IPAddressListEntry[] = await DB.drizzle
			.select({
				id: ipAddress.id,
				ip_address: sql<string>`INET6_NTOA(${ipAddress.ip_address})`,
				flags: ipAddress.flags,
				created_at: ipAddress.created_at,
				updated_at: ipAddress.updated_at,
				user_count: countDistinct(user.id),
				list: {
					id: ipList.id,
					name: ipList.name
				}
			})
			.from(ipAddressSubquery)
			.innerJoin(ipAddress, eq(ipAddress.id, ipAddressSubquery.id))
			.leftJoin(ipList, eq(ipAddress.listId, ipList.id))
			.leftJoin(ipUser, eq(ipAddress.id, ipUser.ipAddressId))
			.leftJoin(user, eq(user.id, ipUser.userId))
			.orderBy(desc(ipAddress.updated_at))
			.groupBy(ipAddress.id);

		const meta: PaginationMeta = {
			rowCount,
			pageSize: limit,
			pageCount: Math.ceil(rowCount / limit)
		};

		if (search.includeUsers) {
			await Promise.all(
				list.map(async (entry) => {
					entry.users = await DB.drizzle
						.select({
							uuid: user.uuid,
							display_name: user.display_name
						})
						.from(user)
						.innerJoin(ipUser, eq(user.id, ipUser.userId))
						.where(eq(ipUser.ipAddressId, entry.id));
				})
			);
		}

		return { list, meta } satisfies Paginated<IPAddressListEntry>;
	}

	static async ipCount(search: IPAddressQuery) {
		const [{ rowCount }] = await DB.drizzle
			.select({ rowCount: count(ipAddress.id).mapWith(Number) })
			.from(ipAddress)
			.leftJoin(ipUser, eq(ipUser.ipAddressId, ipAddress.id))
			.leftJoin(user, eq(user.id, ipUser.userId))
			.where(IPAddressesAdmin.getIpAddressWhere(search));
		return rowCount;
	}

	static getIpAddressWhere(search: IPAddressQuery) {
		const selectList: SQL<unknown>[] = [];

		if (search.ip) {
			selectList.push(sql`${ipAddress.ip_address} = INET6_ATON(${search.ip})`);
		}

		if (search.flags) {
			selectList.push(sql`${ipAddress.flags} & ${Number(search.flags)}`);
		}

		if (search.user) {
			selectList.push(eq(user.uuid, search.user));
		}

		if (search.listId) {
			selectList.push(eq(ipAddress.listId, Number(search.listId)));
		}

		return and(...selectList);
	}
}
