import { and, eq, sql } from 'drizzle-orm';
import { DB, ipAddress, ipList, ipUser } from '../drizzle';
import { CacheBackend } from '../cache-backend';
import { IPFlag } from './types';
import { ensureArray } from '$lib/utils';

export class IPAddresses {
	static flagsList = Object.values(IPFlag).filter((entry) => isNaN(Number(entry))) as string[];

	static async getIpAddressesByFlag(flag: IPFlag | IPFlag[]) {
		const flags = ensureArray(flag);
		const combinedFlag = flags.reduce<number>((num, flag) => num | (1 << flag), 0);
		const ipAddresses = await DB.drizzle
			.select({ ip_address: sql<string>`INET6_NTOA(${ipAddress.ip_address})` })
			.from(ipAddress)
			.where(sql`${ipAddress.flags} & ${combinedFlag}`);
		return ipAddresses.map(({ ip_address }) => ip_address);
	}

	static async getIpAddressesByUserId(userId: number) {
		return DB.drizzle
			.select({
				id: ipAddress.id,
				ip_address: sql<string>`INET6_NTOA(${ipAddress.ip_address})`,
				flags: ipAddress.flags,
				updated_at: ipAddress.updated_at,
				list: {
					id: ipList.id,
					name: ipList.name
				}
			})
			.from(ipAddress)
			.leftJoin(ipUser, eq(ipAddress.id, ipUser.ipAddressId))
			.leftJoin(ipList, eq(ipList.id, ipAddress.listId))
			.where(eq(ipUser.userId, userId));
	}

	static async getCachedIpAddressesByFlag(flag: IPFlag) {
		const existing = await CacheBackend.get<string[]>(`ipAddressList${flag}`);
		if (!existing) {
			const result = await IPAddresses.getIpAddressesByFlag(flag);
			await CacheBackend.set(`ipAddressList${flag}`, result, 60 * 60 * 1000); // 1 hour cache
			return result;
		}
		return existing;
	}

	static async isIpFlagged(address: string, flag: IPFlag) {
		const [entry] = await DB.drizzle
			.select()
			.from(ipAddress)
			.where(
				and(
					sql`${ipAddress.ip_address} = INET6_ATON(${address})`,
					sql`${ipAddress.flags} & ${1 << flag}`
				)
			);
		return !!entry;
	}

	static async storeOrUpdateIpAddress(
		address: string,
		flags: IPFlag[],
		listId?: number,
		userId?: number
	) {
		let updateId: number;
		const combinedFlag = flags.reduce<number>((num, flag) => num | (1 << flag), 0);
		const [existingIp] = await DB.drizzle
			.select()
			.from(ipAddress)
			.where(sql`${ipAddress.ip_address} = INET6_ATON(${address})`);

		if (existingIp) {
			updateId = existingIp.id;
			existingIp.flags = existingIp.flags | combinedFlag;
			existingIp.updated_at = new Date();
			if (listId) {
				existingIp.listId = listId || null;
			}
			await DB.drizzle.update(ipAddress).set(existingIp).where(eq(ipAddress.id, existingIp.id));
		} else {
			const [result] = await DB.drizzle
				.insert(ipAddress)
				.values({
					ip_address: sql`INET6_ATON(${address})`,
					flags: combinedFlag,
					listId: listId || null
				})
				.$returningId();
			updateId = result.id;
		}

		if (userId) {
			const [relationExists] = await DB.drizzle
				.select()
				.from(ipUser)
				.where(and(eq(ipUser.userId, userId), eq(ipUser.ipAddressId, updateId)));

			if (relationExists) {
				return updateId;
			}

			await DB.drizzle.insert(ipUser).values({
				ipAddressId: updateId,
				userId
			});
		}

		return updateId;
	}
}

export { IPFlag };
