export type RequiredPrivileges = (string | string[])[];

/**
 * Check a list of privileges against a list of required privileges.
 * @param list List of privileges
 * @param privileges OR logic, use a nested array for AND
 * @returns Satisfies privileges
 */
export const hasPrivileges = (list: string[], privileges: RequiredPrivileges) =>
	privileges.some((item) => {
		if (Array.isArray(item)) {
			return item.every((sub) => list.includes(sub));
		}
		return list.includes(item);
	});

/**
 * Wait until the `check` function returns a truthy value.
 * @param check Condition to run
 * @param checkInterval Interval to run condition (ms)
 * @param checkTimeout Timeout of the check (ms)
 */
export const waitIsTruthy = (check: () => boolean, checkInterval = 500, checkTimeout = 5000) => {
	let time = 0;
	return new Promise<void>((resolve, reject) => {
		const interval = setInterval(() => {
			if (check.call(null)) {
				clearInterval(interval);
				return resolve();
			}

			if (time > checkTimeout) {
				clearInterval(interval);
				return reject();
			}

			time += checkInterval;
		}, checkInterval);
	});
};

/**
 * Only handle event if the event target is current element
 * @param fn True event handler
 * @returns Wrapped event handler
 */
export function self<TFn extends (event: Event, ...args: Array<unknown>) => void>(fn: TFn) {
	return function (this: EventTarget, ...args: Parameters<TFn>) {
		const event = args[0];
		if (event.target === this) {
			fn?.apply(this, args);
		}
	};
}

/**
 * Ensures that the given input is an array.
 * @param input Array or single value
 * @returns Array
 */
export const ensureArray = <T>(input: T | T[]): T[] => (Array.isArray(input) ? input : [input]);
